#version 330 core

struct LightProperties
{
	vec4 position;
	vec4 attenuation;
	vec4 ambientColor;
	vec4 diffuseColor;
	vec4 specularColor;
};

struct MaterialProperties
{
	vec4 ambientColor;
	vec4 diffuseColor;
	vec4 specularColor;
	float specularExponent;
};

uniform	LightProperties light[2];
uniform MaterialProperties material;
uniform vec3 eyePos;
uniform float farDist;

uniform sampler2D colorTexture;
uniform samplerCubeShadow shadowTexture[2];

in vec2 texCoordout;
smooth in vec3 smooth_normal;
in vec4 vertexPos;
out vec4 out_color;

void main(void)
{
	vec4 texel = texture2D(colorTexture, texCoordout);
	vec3 vertToEye = normalize(eyePos-vertexPos.xyz);
	vec4 color = vec4(0.0);

	for(int i=0; i < 2; i++)
	{
		vec3 lightDir = vertexPos.xyz-light[i].position.xyz;
		float lightDist = length(lightDir);
		lightDir = normalize(lightDir);
		vec3 refl = normalize(reflect(lightDir, smooth_normal));
		float diffuseFactor = max(dot(smooth_normal, -normalize(lightDir)),0);
		float specularFactor = dot(vertToEye, refl);
		specularFactor = max(pow(specularFactor, material.specularExponent),0);

		float totalAttenuation = light[i].attenuation.x + light[i].attenuation.y * lightDist + light[i].attenuation.z * lightDist * lightDist;
		vec4 ambientColor = light[i].ambientColor*material.ambientColor / totalAttenuation;
		vec4 diffuseColor = diffuseFactor*light[i].diffuseColor*material.diffuseColor / totalAttenuation;
		vec4 specularColor = specularFactor*light[i].specularColor*material.specularColor / totalAttenuation;

		if(abs(lightDir.x) > abs(lightDir.y) && abs(lightDir.x) > abs(lightDir.z)){
			lightDir.y = -lightDir.y;
			lightDir.z = -lightDir.z;
		} else if(abs(lightDir.z) > abs(lightDir.y)) {
			lightDir.x = -lightDir.x;
			lightDir.y = -lightDir.y;
		}

		float bias = 0.000005*tan(acos(diffuseFactor));
		float sh = texture(shadowTexture[i], vec4(lightDir,(lightDist / farDist)-bias));
		diffuseColor = diffuseColor * sh;
		specularColor = specularColor * sh;

		color = color + ambientColor + diffuseColor + specularColor;
	}
	out_color =  texel * color;
}