#version 330 core

struct LightProperties
{
	vec4 position;
	vec4 attenuation;
	vec4 ambientColor;
	vec4 diffuseColor;
	vec4 specularColor;
};

struct MaterialProperties
{
	vec4 ambientColor;
	vec4 diffuseColor;
	vec4 specularColor;
	float specularExponent;
};

uniform	LightProperties light[2];
uniform MaterialProperties material;
uniform vec3 eyePos;

uniform mat4 projectionMatrix;
uniform mat4 modelMatrix;
uniform mat4 viewMatrix;
uniform mat3 normalMatrix;

in vec4 in_position;
in vec3 in_normal;
in vec3 in_tangent;
in vec3 in_bitangent;
in vec2 in_texCoord;

out vec2 v_texCoord;
out vec3 v_tsLight1;
out vec3 v_tsLight2;
out vec3 v_tsEye;
out vec4 v_position;
out vec3 v_normal;

void main(void)
{
		v_texCoord = in_texCoord;
		v_normal = normalize(normalMatrix * in_normal);
		vec4 position = modelMatrix * in_position;
		v_position = position;
        // Bring tangent, bitangent and normal to camera space.
		mat3x3 normalMatrixV = mat3x3(viewMatrix * modelMatrix);
        vec3 tangent = normalize(normalMatrixV*in_tangent);
        vec3 bitangent = normalize(normalMatrixV*in_bitangent);
        vec3 normal = normalize(normalMatrixV*in_normal);

        // The inverse brings the light from camera to texture space.
        // tx ty tz lightDirectionx
        // bx by bz * lightDirectiony
        // nx ny nz lightDirectionz
		vec3 lightDir1 = vec3(viewMatrix * vec4(position - light[0].position));
		vec3 lightDir2 = vec3(viewMatrix * vec4(position - light[1].position));
		
		mat3x3 TBN = transpose(mat3x3(tangent, bitangent, normal));

        v_tsLight1 = TBN * lightDir1;
		v_tsLight2 = TBN * lightDir2;


        // Bring vertex in camera space.
        vec4 vertex = viewMatrix * modelMatrix * in_position;
        
        // Calculate eye vector, which is in camera space.
        vec3 eye = -vec3(vertex);
        
        // Like the light, this brings the eye vector to texture space.
		v_tsEye = TBN * eye;
        gl_Position = projectionMatrix*vertex;
}